<?php
/**
 * BudouX MediaWiki extension  version 1.0.1
 *	for details please see: https://www.mediawiki.org/wiki/Extension:BudouX
 *
 * Copyright (c) 2023 Kimagurenote https://kimagurenote.net/
 * License:Apache License 2.0 http://www.apache.org/licenses/LICENSE-2.0
 *
 * Function:
 *	Mediawiki extension to integrate Google BudouX to enable beautiful and readable line breaks for CJK languages.
 *
 * Dependency:
 *	MediaWiki 1.25+
 *	This extension registrated wfLoadExtension() only.
 *	https://www.mediawiki.org/wiki/Manual:Extension_registration
 *	PHP 7.0+ - define with array
 *	https://www.php.net/manual/en/function.define.php
 *	Google BudouX JavaScript module
 *	https://github.com/google/budoux/tree/main/javascript
 *
 * History:
 * 2023.09.26	v1.0.1
 *	support $wgBudouXTags and $wgBudouXExcludes
 * 2023.09.25	v1.0.0
 *	1st version
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2023 Kimagurenote
 * @license Apache-2.0
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

define( 'BUDOUX_SCRIPT_PATH', "https://unpkg.com/budoux/bundle/budoux-ja.min.js" );	// default script path
define( 'BUDOUX_ID', 'BudouX' );	// name & id
define( 'BUDOUX_LANGS', [	// accepted lanuages
	'ja',		// for Japanese
	'zh-hans',	// for Simplified Chinese
	'zh-hant'	// for Traditional Chinese
] );
define( 'BUDOUX_TAGS', [ "p", "dd", "li" ] );	// tags to adapt
//define( 'BUDOUX_DEBUG', 'budoux' );	// for debug

class BudouX {

	/**
	 * add budoux tag
	 */
	private static function addBudouXTag( $tag, $text ) {
		global $wgBudouXLang;

		if( stripos( $text, '<budoux-' ) !== false ) {
			return $text;
		}

		$tags = is_array( $tag ) ? $tag : [ $tag ];

		foreach( $tags as $t ) {
			$text = str_ireplace( "<${t}>", "<${t}><budoux-${wgBudouXLang}>", $text );
			$text = str_ireplace( "</${t}>", "</budoux-${wgBudouXLang}></${t}>", $text );
		}

		return $text;
	}


	/**
	 * @param OutputPage $out
	 *	https://doc.wikimedia.org/mediawiki-core/1.23.7/php/classOutputPage.html
	 * @param Skin $skin
	 * return void
	 * https://www.mediawiki.org/wiki/Manual:Hooks/BeforePageDisplay
	 *
	 * Hook that allows last minute changes to the output page, e.g.
	 * adding of CSS or Javascript by extensions, adding CSP sources.
	 * https://doc.wikimedia.org/mediawiki-core/master/php/OutputPage_8php_source.html#l02917
	 */
	public static function onBeforePageDisplay( OutputPage $out, Skin $skin ) {
		global $wgRequest, $wgBudouXExcludes, $wgContentNamespaces, $wgBudouXLang, $wgBudouXScriptPath, $wgBudouXTags;

		/* get Title */
		$title = $skin->getTitle();

		/* check parameter */
		if( empty( $wgBudouXLang ) or !in_array( $wgBudouXLang, BUDOUX_LANGS ) ) {
			return;
		}

		/* check action */
		if( $wgRequest->getText( 'action' ) ) {
			return;
		}

		/* check page exist */
		if( !$title->exists() ) {
			return;
		}

		/* check redirect */
		if( $out->getRedirect() ) {
			return;
		}

		/* check page title */
		if( !empty( $wgBudouXExcludes ) and is_array( $wgBudouXExcludes ) ) {
			if( in_array( $title->getBaseText(), $wgBudouXExcludes ) ) {
				return;
			}
		}

		/* check namespace */
		$ns = $title->getNamespace();
		if( empty( $wgContentNamespaces ) ) {
			if( $ns != NS_MAIN ) {
				return;
			}
		} elseif ( !in_array( $ns, $wgContentNamespaces, true ) ) {
			return;
		}

		/* add a <script> in the <head> */
		$out->addHeadItem( BUDOUX_ID, '<script src="' . ( $wgBudouXScriptPath ?? BUDOUX_SCRIPT_PATH ) . '"></script>' );

		/* get body HTML */
		$text = $out->getHTML();

		/* add budoux tag */
		$text = self::addBudouXTag( ( $wgBudouXTags ?? BUDOUX_TAGS ), $text );

		/* replace body HTML */
		$out->clearHTML();
		$out->addHTML( $text );

		return;
	}
}
